/**
 * This program has been developed by students from the bachelor Computer Science at
 * Utrecht University within the Software Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */
import { Edge as ReactEdge, NodeProps } from 'reactflow';
import { EntityData, EntityNodeAttributes, LogicData, LogicNodeAttributes, RelationData, RelationNodeAttributes } from '../graphology';

/** Enums for the possible types of query elements */
export enum QueryElementTypes {
  Entity = 'entity',
  Relation = 'relation',
  // Attribute = 'attribute',
  // Function = 'function',
  Logic = 'logic',
}

/** List of possible query element types */
export const possibleTypes: string[] = [
  QueryElementTypes.Entity,
  QueryElementTypes.Relation,
  QueryElementTypes.Logic,
  // QueryElementTypes.Attribute,
  // QueryElementTypes.Function,
];

/** All the possible react flow nodes. */
export type SchemaReactflowEntityNode = NodeProps<EntityNodeAttributes>;
export type SchemaReactflowRelationNode = NodeProps<RelationNodeAttributes>;
export type SchemaReactflowLogicNode = NodeProps<LogicNodeAttributes>;

export type SchemaReactflowNode = SchemaReactflowEntityNode | SchemaReactflowRelationNode | SchemaReactflowLogicNode;
export type SchemaReactflowEdge = ReactEdge<any>;
export type SchemaReactflowElement = SchemaReactflowNode | SchemaReactflowEdge;
export type SchemaReactflowNodeData = EntityData | RelationData | LogicData;

/** Interface for updating the edges. */
export interface updateEdges {
  newEdge: SchemaReactflowEdge | undefined;
  removeEdge: SchemaReactflowEdge | undefined;
}

/**
 * Checks if a node is an entityNode.
 * @param {SchemaReactflowNode} node The node that has to checked.
 * @returns True and casts if the node is an EntityNode.
 */
export function isEntityNode(node: SchemaReactflowNode): node is SchemaReactflowEntityNode {
  return node.type == QueryElementTypes.Entity;
}

/**
 * Checks if a node is a RelationNode.
 * @param {SchemaReactflowNode} node The node that has to checked.
 * @returns True and casts if the node is an RelationNode.
 */
export function isRelationNode(node: SchemaReactflowNode): node is SchemaReactflowRelationNode {
  return node.type == QueryElementTypes.Relation;
}

/**
 * Checks if a node is a LogicNode.
 * @param {SchemaReactflowNode} node The node that has to checked.
 * @returns True and casts if the node is an LogicNode.
 */
export function isLogicNode(node: SchemaReactflowNode): node is SchemaReactflowLogicNode {
  return node.type == QueryElementTypes.Logic;
}
