import React from 'react';
import { Meta, StoryObj } from '@storybook/react';
import EntityFlowElement from './entitypill';
import { configureStore } from '@reduxjs/toolkit';
import { Provider } from 'react-redux';

import { querybuilderSlice, schemaSlice, searchResultSlice } from '@graphpolaris/shared/lib/data-access/store';
import { ReactFlowProvider } from 'reactflow';
import { EntityData, Handles, QueryElementTypes } from '../../../model';

const Component: Meta<typeof EntityFlowElement> = {
  title: 'Querybuilder/Pills/EntityPill',
  component: EntityFlowElement,
  decorators: [
    (story) => (
      <Provider store={Mockstore}>
        <ReactFlowProvider>{story()}</ReactFlowProvider>
      </Provider>
    ),
  ],
};

export default Component;

const Mockstore = configureStore({
  reducer: {
    querybuilder: querybuilderSlice.reducer,
    schema: schemaSlice.reducer,
    searchResults: searchResultSlice.reducer,
  },
});

export const Default: StoryObj<{ data: EntityData }> = {
  args: {
    data: {
      name: 'TestEntity',
      leftRelationHandleId: { nodeId: '1', nodeName: 'string', nodeType: QueryElementTypes.Entity, handleType: Handles.EntityLeft },
      rightRelationHandleId: { nodeId: '2', nodeName: 'string2', nodeType: QueryElementTypes.Entity, handleType: Handles.EntityRight },
      selected: false,
    },
  },
};
