import {
  assignNewGraphQueryResult,
  graphQueryResultSlice,
  querybuilderSlice,
  schemaSlice,
  setSchema,
  visualizationSlice,
} from '../../../data-access/store';
import { configureStore } from '@reduxjs/toolkit';
import { Meta } from '@storybook/react';
import { VisualizationPanel } from '../../components/panel';
import { Provider } from 'react-redux';
import { SchemaUtils } from '../../../schema/schema-utils';
import { bigMockQueryResults, simpleSchemaRaw, smallFlightsQueryResults } from '../../../mock-data';
import { simpleSchemaAirportRaw } from '../../../mock-data/schema/simpleAirportRaw';
import { setActiveVisualization } from '@graphpolaris/shared/lib/data-access/store/visualizationSlice';

const Component: Meta<typeof VisualizationPanel> = {
  title: 'Visualizations/Paohvis',
  component: VisualizationPanel,
  decorators: [
    (story) => (
      <Provider store={Mockstore}>
        <div
          style={{
            width: '100%',
            height: '100vh',
          }}
        >
          {story()}
        </div>
      </Provider>
    ),
  ],
};

const Mockstore = configureStore({
  reducer: {
    schema: schemaSlice.reducer,
    graphQueryResult: graphQueryResultSlice.reducer,
    visualize: visualizationSlice.reducer,
    querybuilder: querybuilderSlice.reducer,
  },
});

export const TestWithData = {
  play: async () => {
    const dispatch = Mockstore.dispatch;
    const schema = SchemaUtils.schemaBackend2Graphology({
      nodes: [
        {
          name: '1',
          attributes: [{ name: 'a', type: 'string' }],
        },
      ],
      edges: [
        {
          name: '12',
          label: '12',
          from: '1',
          to: '1',
          collection: '1c',
          attributes: [{ name: 'a', type: 'string' }],
        },
      ],
    });

    dispatch(setSchema(schema.export()));
    dispatch(
      assignNewGraphQueryResult({
        queryID: '1',
        result: {
          type: 'nodelink',
          payload: {
            nodes: [
              { id: '1/a', label: 'a', attributes: { a: 's1' } },
              { id: '1/b1', label: 'b1', attributes: { a: 's1' } },
              { id: '1/b2', label: 'b2', attributes: { a: 's1' } },
              { id: '1/b3', label: 'b3', attributes: { a: 's1' } },
            ],
            edges: [
              { id: '1c/z1', label: 'z1', from: '1/b1', to: '1/a', attributes: { a: 's1' } },
              // { from: 'b2', to: 'a', attributes: {} },
              // { from: 'b3', to: 'a', attributes: {} },
              { id: '1c/z1', label: 'z1', from: '1/a', to: '1/b1', attributes: { a: 's1' } },
            ],
          },
        },
      }),
    );
    dispatch(setActiveVisualization('PaohVis'));
  },
};

export const TestWithAirportSimple = {
  play: async () => {
    const dispatch = Mockstore.dispatch;
    const schema = SchemaUtils.schemaBackend2Graphology(simpleSchemaRaw);

    dispatch(setSchema(schema.export()));
    dispatch(assignNewGraphQueryResult({ queryID: '1', result: { type: 'nodelink', payload: smallFlightsQueryResults } }));
    dispatch(setActiveVisualization('PaohVis'));
  },
};

export const TestWithAirport = {
  play: async () => {
    const dispatch = Mockstore.dispatch;
    const schema = SchemaUtils.schemaBackend2Graphology(simpleSchemaAirportRaw);

    dispatch(setSchema(schema.export()));
    dispatch(assignNewGraphQueryResult({ queryID: '1', result: { type: 'nodelink', payload: bigMockQueryResults } }));
    dispatch(setActiveVisualization('PaohVis'));
  },
};

export default Component;
